@@ -14,7 +14,7 @@ pub struct Eu4SaveMeta {
     pub game: Option<GameState>,
 }
 
-#[derive(Debug, Clone, Deserialize, Serialize)]
+#[derive(Debug, Clone, JominiDeserialize, Serialize)]
 pub struct Meta {
     pub campaign_id: String,
     pub save_game: String,
@@ -22,19 +22,31 @@ pub struct Meta {
     pub displayed_country_name: String,
     pub campaign_length: i32,
     pub date: Eu4Date,
-    #[serde(default)]
+    #[jomini(default)]
     pub is_ironman: bool,
-    #[serde(default, alias = "multi_player")]
+    #[jomini(default, alias = "multi_player")]
+    #[serde(alias = "multi_player")]
     pub multiplayer: bool,
     pub not_observer: bool,
-    #[serde(default)]
+    #[jomini(default)]
     pub dlc_enabled: Vec<String>,
+    #[jomini(default)]
+    pub mod_enabled: Vec<String>,
+    #[jomini(default)]
+    pub mods_enabled_names: Vec<ModName>,
+    #[jomini(take_last)]
     pub checksum: String,
     pub savegame_version: SavegameVersion,
-    #[serde(default)]
+    #[jomini(default)]
     pub is_random_new_world: bool,
 }
 
+#[derive(Debug, Clone, Deserialize, Serialize)]
+pub struct ModName {
+    pub filename: String,
+    pub name: String,
+}
+
 #[derive(Debug, Clone, Deserialize)]
 #[cfg_attr(feature = "serialize", derive(Serialize))]
 pub struct Eu4Save {
@@ -58,7 +70,8 @@ pub struct GameState {
     pub economic_hegemon: Option<Hegemon>,
     pub religion_instance_data: HashMap<String, ReligionInstanceDatum>,
     pub empire: Option<HRE>,
-    pub countries: HashMap<CountryTag, Country>,
+    #[jomini(default, deserialize_with = "deserialize_vec_pair")]
+    pub countries: Vec<(CountryTag, Country)>,
     pub provinces: HashMap<ProvinceId, Province>,
     pub income_statistics: LedgerData,
     pub nation_size_statistics: LedgerData,
@@ -68,6 +81,7 @@ pub struct GameState {
     pub active_wars: Vec<ActiveWar>,
     #[jomini(duplicated, alias = "previous_war")]
     pub previous_wars: Vec<PreviousWar>,
+    #[jomini(default)]
     pub achievement_ok: bool,
     #[jomini(default)]
     pub achievement: Vec<i32>,
@@ -75,6 +89,7 @@ pub struct GameState {
     pub completed_achievements: Vec<i32>,
     #[jomini(alias = "gameplaysettings")]
     pub gameplay_settings: GameplaySettings,
+    pub diplomacy: Diplomacy,
 }
 
 #[derive(Debug, Clone, Deserialize, Serialize, PartialEq)]
@@ -139,6 +154,14 @@ pub struct ReligionInstanceDatum {
     pub defender: Option<CountryTag>,
     #[serde(default)]
     pub defender_date: Option<Eu4Date>,
+    #[serde(default)]
+    pub papacy: Option<Papacy>,
+}
+
+#[derive(Debug, Clone, Deserialize)]
+#[cfg_attr(feature = "serialize", derive(Serialize))]
+pub struct Papacy {
+    pub controller: CountryTag,
 }
 
 #[derive(Debug, Clone, JominiDeserialize)]
@@ -151,16 +174,14 @@ pub struct HRE {
     pub electors: Vec<CountryTag>,
 }
 
-#[derive(Debug, Clone, Copy, Deserialize, PartialEq)]
-#[cfg_attr(feature = "serialize", derive(Serialize))]
+#[derive(Debug, Clone, Copy, Deserialize, Serialize, PartialEq)]
 pub enum TaxManpowerModifier {
     Historical,
     Random,
     Equal,
 }
 
-#[derive(Debug, Clone, Copy, Deserialize, PartialEq)]
-#[cfg_attr(feature = "serialize", derive(Serialize))]
+#[derive(Debug, Clone, Copy, Deserialize, Serialize, PartialEq)]
 pub enum GameDifficulty {
     VeryEasy,
     Easy,
@@ -169,8 +190,7 @@ pub enum GameDifficulty {
     VeryHard,
 }
 
-#[derive(Debug, Clone)]
-#[cfg_attr(feature = "serialize", derive(Serialize))]
+#[derive(Debug, Clone, Serialize)]
 pub struct GameplayOptions {
     pub difficulty: GameDifficulty,
     pub tax_manpower_modifier: TaxManpowerModifier,
@@ -244,6 +264,8 @@ pub struct Province {
     pub modifiers: Vec<Modifier>,
     #[jomini(default)]
     pub history: ProvinceHistory,
+    #[jomini(default = "default_true")]
+    pub ub: bool,
 }
 
 #[derive(Debug, Clone, Deserialize)]
@@ -282,7 +304,7 @@ pub enum ProvinceEvent {
 #[derive(Debug, Clone, Deserialize)]
 #[cfg_attr(feature = "serialize", derive(Serialize))]
 pub struct ControllerEvent {
-    tag: CountryTag,
+    pub tag: CountryTag,
 }
 
 #[derive(Debug, Clone)]
@@ -354,6 +376,8 @@ pub struct Country {
     pub innovativeness: f32,
     #[jomini(default)]
     pub religious_unity: f32,
+    #[jomini(default)]
+    pub church: Option<CountryChurch>,
     pub initialized_rivals: bool,
     pub national_focus: Option<String>,
     pub recalculate_strategy: bool,
@@ -414,6 +438,8 @@ pub struct Country {
     pub mercenary_companries: Vec<MercenaryCompany>,
     pub monarch: Option<ObjId>,
     pub heir: Option<ObjId>,
+    #[jomini(duplicated, alias = "leader")]
+    pub leaders: Vec<ObjId>,
     #[jomini(duplicated, alias = "previous_monarch")]
     pub previous_monarchs: Vec<ObjId>,
 }
@@ -481,6 +507,18 @@ impl CountryEvent {
             _ => None,
         }
     }
+
+    pub fn as_leader(&self) -> Option<&Leader> {
+        self.as_monarch()
+            .and_then(|x| x.leader.as_ref())
+            .or_else(|| {
+                if let CountryEvent::Leader(x) = &self {
+                    Some(x)
+                } else {
+                    None
+                }
+            })
+    }
 }
 
 #[derive(Debug, Clone, Deserialize, Serialize)]
@@ -529,6 +567,7 @@ pub struct Leader {
     #[serde(default)]
     pub siege: u16,
     pub monarch_id: Option<ObjId>,
+    pub personality: Option<String>,
 
     // While activation and id can be none, it is so rare that there
     // is a test case for it to prevent regression.
@@ -559,15 +598,25 @@ pub struct CountryColors {
     pub country_color: Vec<u8>,
 }
 
+#[derive(Debug, Clone, JominiDeserialize)]
+#[cfg_attr(feature = "serialize", derive(Serialize))]
+pub struct CountryChurch {
+    pub power: f32,
+    #[jomini(duplicated, alias = "aspect")]
+    pub aspects: Vec<String>,
+}
+
 #[derive(Debug, Clone, Deserialize)]
 #[cfg_attr(feature = "serialize", derive(Serialize))]
 pub struct Loan {
     pub id: ObjId,
     pub lender: String,
     pub interest: f32,
+    #[serde(default)]
     pub fixed_interest: bool,
     pub amount: i32,
     pub expiry_date: Eu4Date,
+    #[serde(default)]
     pub spawned: bool,
 }
 
@@ -775,3 +824,26 @@ pub struct BattleSide {
     #[serde(deserialize_with = "empty_string_is_none")]
     pub commander: Option<String>,
 }
+
+fn default_true() -> bool {
+    true
+}
+
+#[derive(Debug, Clone, JominiDeserialize)]
+#[cfg_attr(feature = "serialize", derive(Serialize))]
+pub struct Diplomacy {
+    #[jomini(duplicated, default, alias = "dependency")]
+    pub dependencies: Vec<DiplomacyDependency>,
+}
+
+#[derive(Debug, Clone, JominiDeserialize)]
+#[cfg_attr(feature = "serialize", derive(Serialize))]
+pub struct DiplomacyDependency {
+    pub first: CountryTag,
+    pub second: CountryTag,
+    #[jomini(default)]
+    pub start_date: Option<Eu4Date>,
+    #[jomini(default)]
+    pub end_date: Option<Eu4Date>,
+    pub subject_type: String,
+}
