@@ -5,7 +5,7 @@ use crate::{
     },
     ProvinceId, TagResolver,
 };
-use crate::{CountryTag, Eu4Date};
+use crate::{CountryTag, Eu4Date, PdsDate};
 use once_cell::sync::OnceCell;
 use serde::{Deserialize, Serialize};
 use std::collections::{HashMap, HashSet};
@@ -135,6 +135,7 @@ pub struct CountryManaSpend {
     pub effect: i32,
     pub minority_expulsion: i32,
     pub other: i32,
+    pub add_tribal_land: i32,
 }
 
 #[derive(Debug, PartialEq, Clone, Copy, Serialize, Deserialize)]
@@ -189,8 +190,8 @@ pub struct NationEvents {
     pub initial: CountryTag,
 
     /// The latest tag that a country achieved. If DMS -> IRE but then
-    // IRE is annexed by SCO which culture shifts to form IRE then both
-    // initial tags of SCO and DMS will report a latest tag of IRE
+    /// IRE is annexed by SCO which culture shifts to form IRE then both
+    /// initial tags of SCO and DMS will report a latest tag of IRE
     pub latest: CountryTag,
 
     /// The tag which the history of this country is stored under. For
@@ -239,16 +240,80 @@ pub struct ResolvedWarParticipants {
     pub participants: Vec<ResolvedWarParticipant>,
 }
 
+#[derive(Debug, Clone)]
+#[cfg_attr(feature = "serialize", derive(Serialize))]
+pub struct SaveCountry<'a> {
+    pub id: usize,
+    pub tag: CountryTag,
+    pub country: &'a Country,
+}
+
+#[derive(Debug, Clone, Copy)]
+struct TagId {
+    id: usize,
+    tag: CountryTag,
+}
+
+#[derive(Debug, Clone, Serialize)]
+pub struct Inheritance {
+    pub start_t0_year: i16,
+    pub end_t0_year: i16,
+    pub start_t1_year: i16,
+    pub end_t1_year: i16,
+    pub start_t2_year: i16,
+    pub end_t2_year: i16,
+    pub inheritance_value: u8,
+    pub subtotal: i64,
+    pub calculations: Vec<Calculation>,
+}
+
+#[derive(Debug, Clone, Copy, Serialize)]
+pub enum TagDependency {
+    Independent,
+    Dependent(CountryTag),
+}
+
+#[derive(Debug, Clone, Serialize)]
+pub struct Calculation {
+    pub name: String,
+    pub value: i64,
+    pub dependency: TagDependency,
+}
+
+impl Calculation {
+    pub fn new(name: &str, value: i64, dependency: TagDependency) -> Self {
+        Self {
+            name: String::from(name),
+            value,
+            dependency,
+        }
+    }
+}
+
 #[derive(Debug)]
 pub struct Query {
     save: Eu4Save,
+    tag_ids: Vec<TagId>,
+    tag_lookup: HashMap<CountryTag, TagId>,
     buildings: OnceCell<HashSet<String>>,
 }
 
 impl Query {
     pub fn from_save(save: Eu4Save) -> Self {
+        let tag_ids: Vec<_> = save
+            .game
+            .countries
+            .iter()
+            .enumerate()
+            .map(|(i, (tag, _))| TagId { id: i, tag: *tag })
+            .collect();
+
+        let tag_lookup = tag_ids.iter().map(|id| (id.tag, *id)).collect();
+
         Query {
             save,
+            tag_ids,
+            tag_lookup,
             buildings: OnceCell::default(),
         }
     }
@@ -257,6 +322,37 @@ impl Query {
         &self.save
     }
 
+    pub fn countries(&self) -> impl Iterator<Item = SaveCountry> + '_ {
+        self.save
+            .game
+            .countries
+            .iter()
+            .zip(self.tag_ids.iter())
+            .map(|((_, country), tag_id)| SaveCountry {
+                id: tag_id.id,
+                tag: tag_id.tag,
+                country,
+            })
+    }
+
+    pub fn save_country(&self, tag: &CountryTag) -> Option<SaveCountry> {
+        self.tag_lookup.get(tag).and_then(|tag_id| {
+            self.save
+                .game
+                .countries
+                .get(tag_id.id)
+                .map(|(_, country)| SaveCountry {
+                    id: tag_id.id,
+                    tag: tag_id.tag,
+                    country,
+                })
+        })
+    }
+
+    pub fn country(&self, tag: &CountryTag) -> Option<&Country> {
+        self.save_country(tag).map(|x| x.country)
+    }
+
     /// Returns a set of the names of the players who participated in a playthrough.
     /// May be blank for single player run through where the player is detected as
     /// "Player". No guarantees are given as to the state of each player's country.
@@ -275,7 +371,63 @@ impl Query {
     /// - The names of the players in the session
     /// - A list of all prior tags that country had been
     pub fn player_histories(&self, nation_events: &[NationEvents]) -> Vec<PlayerHistory> {
-        calc_histories(&self.save, nation_events)
+        let save = self.save();
+        let mut result = Vec::with_capacity(save.game.players_countries.len());
+        let players = players(save);
+        let mut leftovers = Vec::new();
+        for (tag, country) in save.game.countries.iter().filter(|(_, c)| c.was_player) {
+            let tag = *tag;
+            let tag_players: Vec<_> = players
+                .iter()
+                .filter(|x| x.tag == tag)
+                .map(|x| x.name.clone())
+                .collect();
+            let history = nation_events
+                .iter()
+                .find(|x| x.stored == tag)
+                .cloned()
+                .unwrap_or_else(|| NationEvents {
+                    initial: tag,
+                    latest: tag,
+                    stored: tag,
+                    events: Vec::new(),
+                });
+
+            let no_players = tag_players.is_empty();
+            let history = PlayerHistory {
+                history,
+                is_human: country.human,
+                player_names: tag_players,
+            };
+
+            if country.was_player && no_players {
+                leftovers.push(history);
+            } else {
+                result.push(history);
+            }
+        }
+
+        // Only for ironman will we try and resolve "release and play as" as the
+        // save does not often paint an accurate picture of these transitions. And
+        // we need to track these for achievements like spaghetti western.
+        if result.len() == 1 && save.meta.is_ironman {
+            for x in &mut result {
+                if x.is_human && x.player_names.len() == 1 && x.history.stored == x.history.latest {
+                    let country = self.country(&x.history.stored);
+                    let rpa = country.map_or(false, |x| x.has_switched_nation);
+                    let alive = country.map_or(false, |x| x.num_of_cities > 0);
+                    if rpa && alive {
+                        if let Some(end) = leftovers.pop() {
+                            x.history.initial = end.history.initial;
+                        }
+                    }
+                }
+            }
+        }
+
+        result.append(&mut leftovers);
+
+        result
     }
 
     /// Calculates the major events that befell countries (annexations, appearances, and tag switches)
@@ -301,6 +453,114 @@ impl Query {
         TagResolver::create(nation_events)
     }
 
+    fn inherit_subtotal(&self, country: &SaveCountry) -> (i64, Vec<Calculation>) {
+        let hre_ruler = self
+            .save()
+            .game
+            .empire
+            .as_ref()
+            .and_then(|x| x.emperor)
+            .and_then(|x| self.country(&x))
+            .and_then(|x| x.monarch.as_ref())
+            .map(|x| i64::from(x.id))
+            .unwrap_or_default();
+
+        let papacy_controller = self
+            .save()
+            .game
+            .religion_instance_data
+            .get("catholic")
+            .and_then(|x| x.papacy.as_ref())
+            .and_then(|x| self.save_country(&x.controller));
+
+        let papacy_tag = papacy_controller
+            .as_ref()
+            .map(|x| x.tag)
+            .unwrap_or_else(|| "---".parse().unwrap());
+        let papacy_id = papacy_controller
+            .as_ref()
+            .map(|x| x.id as i64)
+            .unwrap_or_default();
+
+        let ruler = country
+            .country
+            .monarch
+            .as_ref()
+            .map(|x| i64::from(x.id))
+            .unwrap_or_default();
+
+        let previous_rulers = country
+            .country
+            .previous_monarchs
+            .iter()
+            .map(|x| i64::from(x.id))
+            .sum::<i64>();
+
+        let capital_province = i64::from(country.country.capital.as_u16());
+
+        let provinces = i64::from(country.country.num_of_cities);
+
+        #[rustfmt::skip]
+        let calculations = vec![
+            Calculation::new("Nation ID", country.id as i64, TagDependency::Dependent(country.tag)),
+            Calculation::new("HRE Ruler ID", hre_ruler, TagDependency::Independent),
+            Calculation::new("Curia Controller Nation ID", papacy_id, TagDependency::Dependent(papacy_tag)),
+            Calculation::new("Ruler ID", ruler, TagDependency::Dependent(country.tag)),
+            Calculation::new("Previous Ruler IDs", previous_rulers, TagDependency::Dependent(country.tag)),
+            Calculation::new("Capital Province", capital_province, TagDependency::Dependent(country.tag)),
+            Calculation::new("Owned Provinces", provinces, TagDependency::Dependent(country.tag)),
+        ];
+
+        let raw = hre_ruler
+            + papacy_id
+            + ruler
+            + previous_rulers
+            + capital_province
+            + provinces
+            + country.id as i64;
+
+        (raw, calculations)
+    }
+
+    pub fn inherit(&self, country: &SaveCountry) -> Inheritance {
+        let (subtotal, calculations) = self.inherit_subtotal(country);
+
+        let year = i64::from(self.save().meta.date.year());
+        let inheritance_value = (subtotal + year) % 100;
+
+        let t0_mod = (0 - inheritance_value) % 100;
+        let t1_mod = (75 - inheritance_value) % 100;
+        let t2_mod = (80 - inheritance_value) % 100;
+
+        // end date < year => +100
+        // end date > year + 100 => -100
+        let t0_offset = if year + t0_mod + 74 < year { 100 } else { 0 };
+
+        let t1_offset = if year + t1_mod + 4 < year { 100 } else { 0 };
+
+        let t2_offset = if year + t2_mod + 19 < year { 100 } else { 0 };
+
+        // dbg!((century, t0_offset, t0_mod));
+        let start_t0_year = year + t0_mod + t0_offset;
+        let end_t0_year = year + t0_mod + t0_offset + 74;
+        let start_t1_year = year + t1_mod + t1_offset;
+        let end_t1_year = year + t1_mod + t1_offset + 4;
+        let start_t2_year = year + t2_mod + t2_offset;
+        let end_t2_year = year + t2_mod + t2_offset + 19;
+
+        Inheritance {
+            start_t0_year: start_t0_year as i16,
+            end_t0_year: end_t0_year as i16,
+            start_t1_year: start_t1_year as i16,
+            end_t1_year: end_t1_year as i16,
+            start_t2_year: start_t2_year as i16,
+            end_t2_year: end_t2_year as i16,
+            inheritance_value: inheritance_value as u8,
+            subtotal,
+            calculations,
+        }
+    }
+
     pub fn resolved_war_participants(
         &self,
         tag_resolver: &TagResolver,
@@ -425,10 +685,7 @@ impl Query {
     }
 
     pub fn country_tag_hex_color(&self, country_tag: &CountryTag) -> Option<String> {
-        self.save
-            .game
-            .countries
-            .get(&country_tag)
+        self.country(country_tag)
             .map(|x| self.country_color_to_hex(x))
     }
 
@@ -469,7 +726,7 @@ impl Query {
             .countries
             .iter()
             .filter(|(_, country)| country.num_of_cities > 0)
-            .map(|(&tag, country)| (tag, self.country_income_breakdown(country)))
+            .map(|(tag, country)| (*tag, self.country_income_breakdown(country)))
             .collect()
     }
 
@@ -479,7 +736,7 @@ impl Query {
             .countries
             .iter()
             .filter(|(_, country)| country.num_of_cities > 0)
-            .map(|(&tag, country)| (tag, self.country_expense_breakdown(country)))
+            .map(|(tag, country)| (*tag, self.country_expense_breakdown(country)))
             .collect()
     }
 
@@ -489,7 +746,7 @@ impl Query {
             .countries
             .iter()
             .filter(|(_, c)| c.ledger.totalexpensetable.iter().any(|&x| x > 0.0))
-            .map(|(&tag, country)| (tag, self.country_total_expense_breakdown(country)))
+            .map(|(tag, country)| (*tag, self.country_total_expense_breakdown(country)))
             .collect()
     }
 
@@ -546,6 +803,12 @@ impl Query {
     }
 
     fn mana_spent_indexed(&self, data: &[(i32, i32)]) -> CountryManaSpend {
+        let offset = if self.save().meta.savegame_version.second >= 31 {
+            1
+        } else {
+            0
+        };
+
         let force_march = find_index(8, data) + find_index(45, data);
         CountryManaSpend {
             buy_idea: find_index(0, data),
@@ -573,29 +836,34 @@ impl Query {
             reduce_we: find_index(22, data),
             boost_faction: find_index(23, data),
             raise_war_taxes: find_index(24, data),
-            buy_native_advancement: find_index(25, data),
-            increse_tariffs: find_index(26, data),
-            promote_merc: find_index(27, data),
-            decrease_tariffs: find_index(28, data),
-            move_trade_port: find_index(29, data),
-            create_trade_post: find_index(30, data),
-            siege_sorties: find_index(31, data),
-            buy_religious_reform: find_index(32, data),
-            set_primary_culture: find_index(33, data),
-            add_accepted_culture: find_index(34, data),
-            remove_accepted_culture: find_index(35, data),
-            strengthen_government: find_index(36, data),
-            boost_militarization: find_index(37, data),
-            artillery_barrage: find_index(39, data),
-            establish_siberian_frontier: find_index(40, data),
-            government_interaction: find_index(41, data),
-            naval_barrage: find_index(43, data),
+            buy_native_advancement: if offset != 0 { 0 } else { find_index(25, data) },
+            increse_tariffs: find_index(26 - offset, data),
+            promote_merc: find_index(27 - offset, data),
+            decrease_tariffs: find_index(28 - offset, data),
+            move_trade_port: find_index(29 - offset, data),
+            create_trade_post: find_index(30 - offset, data),
+            siege_sorties: find_index(31 - offset, data),
+            buy_religious_reform: find_index(32 - offset, data),
+            set_primary_culture: find_index(33 - offset, data),
+            add_accepted_culture: find_index(34 - offset, data),
+            remove_accepted_culture: find_index(35 - offset, data),
+            strengthen_government: find_index(36 - offset, data),
+            boost_militarization: find_index(37 - offset, data),
+            artillery_barrage: find_index(39 - offset, data),
+            establish_siberian_frontier: find_index(40 - offset, data),
+            government_interaction: find_index(41 - offset, data),
+            naval_barrage: find_index(43 - offset, data),
+            add_tribal_land: if offset != 1 {
+                0
+            } else {
+                find_index(44 - offset, data)
+            },
             create_leader: find_index(46, data),
             enforce_culture: find_index(47, data),
             effect: find_index(48, data),
             minority_expulsion: find_index(49, data),
-            other: find_index(38, data)
-                + find_index(42, data)
+            other: find_index(38 - offset, data)
+                + find_index(42 - offset, data)
                 + find_index(44, data)
                 + data
                     .iter()
@@ -700,7 +968,8 @@ fn nation_events(save: &Eu4Save, province_owners: &ProvinceOwners) -> Vec<Nation
     let mut nation_events = HashMap::with_capacity(save.game.countries.len());
     let mut all_switches = Vec::with_capacity(save.game.countries.len());
     let mut initial_to_stored = HashMap::with_capacity(save.game.countries.len());
-    for (&tag, country) in &save.game.countries {
+    for (tag, country) in &save.game.countries {
+        let tag = *tag;
         let mut country_tag_switches = Vec::new();
 
         for (date, events) in &country.history.events {
@@ -921,36 +1190,6 @@ fn province_owners(save: &Eu4Save) -> ProvinceOwners {
     ProvinceOwners { initial, changes }
 }
 
-fn calc_histories(save: &Eu4Save, nation_events: &[NationEvents]) -> Vec<PlayerHistory> {
-    let mut result = Vec::with_capacity(save.game.players_countries.len());
-    let players = players(save);
-    for (&tag, country) in save.game.countries.iter().filter(|(_, c)| c.was_player) {
-        let tag_players: Vec<_> = players
-            .iter()
-            .filter(|x| x.tag == tag)
-            .map(|x| x.name.clone())
-            .collect();
-        let history = nation_events
-            .iter()
-            .find(|x| x.stored == tag)
-            .cloned()
-            .unwrap_or_else(|| NationEvents {
-                initial: tag,
-                latest: tag,
-                stored: tag,
-                events: Vec::new(),
-            });
-
-        result.push(PlayerHistory {
-            history,
-            is_human: country.human,
-            player_names: tag_players,
-        })
-    }
-
-    result
-}
-
 fn players(save: &Eu4Save) -> Vec<Player> {
     let mut players = Vec::new();
     for entry in save.game.players_countries.chunks_exact(2) {
